package de.jtem.application.vr.tools;

import java.awt.Color;

import de.jreality.geometry.PointSetFactory;
import de.jreality.scene.Appearance;
import de.jreality.scene.SceneGraphComponent;
import de.jreality.shader.CommonAttributes;
import de.jreality.tools.DragEventTool;
import de.jreality.tools.PointDragEvent;
import de.jreality.tools.PointDragListener;

public class EditablePointSet implements PointDragListener {

	/**
	 * the base scene graph component - contains the toolCmp
	 * and optionally other children, like a line set connecting the points
	 */
	private SceneGraphComponent cmp;

	/**
	 * the tool component - contains the DraggingTool and the PointSet
	 */
	private SceneGraphComponent toolCmp;
	
	/**
	 * the appearance of the draggable points
	 */
	private Appearance pointsApp;
	
	/**
	 * the factory to handle the pointSet geometry
	 */
	private PointSetFactory psf;

	/**
	 * the current coordinates of the points
	 */
	private double[][] pointCoordinates;

	/**
	 * a tool that reports if points/lines/faces
	 * are dragged (with the mouse or with the wand)
	 */
	private DragEventTool dragEventTool;

	public EditablePointSet(double[][] initialPoints) {
		
		pointCoordinates = initialPoints;

		psf = new PointSetFactory();
		psf.setVertexCount(pointCoordinates.length);
		psf.setVertexCoordinates(pointCoordinates);
		psf.update();
		
		// create the base component
		cmp = new SceneGraphComponent();
		cmp.setName("editable PointSet");
		
		// create the tool component
		toolCmp = new SceneGraphComponent();
		toolCmp.setName("tool cmp");
		
		// set the point set as the geometry
		toolCmp.setGeometry(psf.getPointSet());
		
		// enable point picking and increase size of the points
		pointsApp = new Appearance();
		pointsApp.setAttribute(CommonAttributes.POINT_SHADER+"."+CommonAttributes.POINT_RADIUS, .1);
		pointsApp.setAttribute(CommonAttributes.POINT_SHADER+"."+CommonAttributes.DIFFUSE_COLOR, Color.yellow);
		pointsApp.setAttribute(CommonAttributes.POINT_SHADER+"."+CommonAttributes.PICKABLE, true);
		toolCmp.setAppearance(pointsApp);
		
		// attach the tool component to the base component
		cmp.addChild(toolCmp);
		
		// create the dragging tool and attach it to the tool component
		dragEventTool = new DragEventTool();
		toolCmp.addTool(dragEventTool);

		// we register as a point drag listener:
		dragEventTool.addPointDragListener(this);

	}
	
	public void pointDragged(PointDragEvent e) {
		System.out.println("vertex "+e.getIndex()+" is dragged to x="+e.getX()+" y="+e.getY()+" z="+e.getZ());
	}
	
	public void pointDragEnd(PointDragEvent e) {
		// we ignore this - nothing special to do
	}

	public void pointDragStart(PointDragEvent e) {
		// we ignore this - nothing special to do
	}

	/**
	 * get the base component of this editable point set
	 * @return the base cmp
	 */
	public SceneGraphComponent getComponent() {
		return cmp;
	}
	
}
